@@ -2,10 +2,10 @@
  *  cache-mysql.c
  *  mod_musicindex
  *
- *  $Id: cache-mysql.c 899 2009-12-04 13:55:29Z varenet $
+ *  $Id: cache-mysql.c 943 2010-06-06 17:39:54Z varenet $
  *
  *  Created by Thibaut VARENE on Tue Feb 22 2005.
- *  Copyright (c) 2005,2007,2009 Thibaut VARENE
+ *  Copyright (c) 2005,2007,2009-2010 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -47,11 +47,10 @@
  * original directory, for cache up-to-date-ness checking.
  *
  * @todo clean error handler when data corruption happens.
- * @todo optimize connexions
  *
  * @author Thibaut Varene
- * @version $Revision: 899 $
- * @date 2005,2007,2009
+ * @version $Revision: 943 $
+ * @date 2005,2007,2009-2010
  */
 
 #include "playlist.h"
@@ -72,7 +71,7 @@
 #define SQL_SMAX_H	256			/**< max length for host */
 #define SUBSTRINGIFY(x)	STRINGIFY(x)
 #define STRINGIFY(x)	#x
-#define CACHE_VERS	1
+#define CACHE_VERS	2
 #define AINC_OVFLERR	1062			/**< AUTO_INCREMENT overflow error code: 1062 == duplicate entry */
 
 /* http://dev.mysql.com/doc/mysql/en/c.html */
@@ -143,13 +142,16 @@ mysql_cache_new_ent(apr_pool_t *pool, MYSQL_ROW mysql_row)
  * SQL_SMAX_UDP and SQL_SMAX_H.
  * We clean up any existing table if they don't look like what we expect.
  *
+ * @todo add a table prefix, so that all tables with that prefix
+ *	will be under module's control
+ *
  * @param s Apache server_rec struct to handle log writings.
  * @param cache_setup configuration string in which the path to the cache root can be found.
  *
  * @return 0 on succes, -1 otherwise.
  */
 static int
-mysql_cache_init(server_rec *s, const char *const cache_setup)
+mysql_cache_init(server_rec *s, const char *const setup_string)
 {
 	MYSQL		*mysql;
 	MYSQL_RES	*mysql_res = NULL;
@@ -168,8 +170,12 @@ mysql_cache_init(server_rec *s, const char *const cache_setup)
 	const char const create_tdirs[] = "CREATE TABLE `" TABLE_DIRS "` ("
 			"`id` SMALLINT UNSIGNED SERIAL DEFAULT VALUE,"	/* 65535 dirs should be plenty enough :P */
 			"`timestamp` INT UNSIGNED,"
+#if MYSQL_VERSION_ID > 50003	/* MySQL 5.0.3 and above support up to 65,535 wide CHAR columns */
+			"`fullpath` VARCHAR(" SUBSTRINGIFY(MAX_PATHNAME) ") CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,"
+			"INDEX `check` (`fullpath`))"
+#else	/* earlier versions stop at 255. No point in creating a fulltext index, we're not MATCH()'ing strings */
 			"`fullpath` TEXT CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,"
-			"FULLTEXT INDEX `check` (`fullpath`))"
+#endif
 			"CHARACTER SET = utf8,"
 			"COLLATE = utf8_bin";
 	
@@ -180,7 +186,7 @@ mysql_cache_init(server_rec *s, const char *const cache_setup)
 	const char const create_tfiles[] = "CREATE TABLE `" TABLE_FILES "` ("
 			"`id` MEDIUMINT UNSIGNED SERIAL DEFAULT VALUE,"	/* 16,777,215 files. Call me when you overflow! */
 			"`pid` SMALLINT UNSIGNED NOT NULL,"
-			"`filetype` TINYINT NOT NULL,"	/* this should be an enum, but I can't figure out how to do it without hardcoding values */
+			"`filetype` TINYINT NOT NULL,"	/* XXX this should be an enum, but I can't figure out how to do it without hardcoding values */
 			"`flags` TINYINT UNSIGNED NOT NULL,"
 			"`track` TINYINT UNSIGNED NOT NULL,"
 			"`posn` TINYINT UNSIGNED NOT NULL,"
@@ -215,11 +221,11 @@ mysql_cache_init(server_rec *s, const char *const cache_setup)
 	mysql_params.ready = 0;
 	
 	/* first, check whether we have a passwd */
-	temp = strchr(cache_setup, ':');
+	temp = strchr(setup_string, ':');
 	
 	if (temp) {
 		/* copy the passwd */
-		len = strchr(cache_setup, '@') - (temp+1);
+		len = strchr(setup_string, '@') - (temp+1);
 		if (len >= SQL_SMAX_UPD)
 			goto err1;
 		
@@ -227,25 +233,25 @@ mysql_cache_init(server_rec *s, const char *const cache_setup)
 		mysql_params.pass[len] = '\0';
 
 		/* store user field length */
-		len = temp - (cache_setup);
+		len = temp - (setup_string);
 	}
 	else {
 		/* NULLify for safety */
 		mysql_params.pass[0] = '\0';
 
 		/* store user field length */
-		len = strchr(cache_setup, '@') - (cache_setup);
+		len = strchr(setup_string, '@') - (setup_string);
 	}
 	
 	/* copy the user field anyway */
 	if (len >= SQL_SMAX_UPD)
 		goto err1;
 	
-	strncpy(mysql_params.user, cache_setup, len);
+	strncpy(mysql_params.user, setup_string, len);
 	mysql_params.user[len] = '\0';
 	
 	/* as well as the host field */
-	temp = strchr(cache_setup, '@')+1;
+	temp = strchr(setup_string, '@')+1;
 	len = strchr(temp, '/') - temp;
 	
 	if (len >= SQL_SMAX_H)
@@ -379,7 +385,11 @@ err2:
 	mysql_close(mysql);
 err1:
 	if (!mysql_params.ready)
-		mi_serror("Likely length overflow in configuration fields");
+		mi_serror("Likely length overflow in configuration fields: "
+			  "user: %u/%u, pass: %u/%u, db: %u/%u, host: %u/%u",
+			  strlen(mysql_params.user), SQL_SMAX_UPD, strlen(mysql_params.pass),
+			  SQL_SMAX_UPD, strlen(mysql_params.db), SQL_SMAX_UPD,
+			  strlen(mysql_params.host), SQL_SMAX_H);
 	return ret;
 
 }
@@ -403,8 +413,7 @@ mysql_cache_trunc_tables(request_rec *r, MYSQL *mysql)
 	
 	if (mysql_query(mysql, truncate_tfiles))
 		goto exit;
-	if (mysql_query(mysql, truncate_tdirs))
-		goto exit;
+	mysql_query(mysql, truncate_tdirs);
 	
 exit:
 	if (mysql_errno(mysql))
@@ -575,6 +584,7 @@ mysql_cache_check_dir(request_rec *r, const mu_config *const conf,
 	
 	mysql_real_escape_string(mysql, sqlstr, path, strlen(path));
 
+	/* XXX MEM OPTIM: asprintf() */
 	query = apr_psprintf(r->pool, "SELECT timestamp FROM `" TABLE_DIRS "` WHERE `fullpath`='%s'", sqlstr);
 	free(sqlstr);
 
@@ -624,7 +634,6 @@ error:
  * the directory level (see #mysql_cache_check_dir() and #mysql_cache_dircontents()).
  *
  * @warning @b MUST close in file handler on success.
- * @bug will generate too many connection/disconnections
  *
  * @param r Apache request_rec struct to handle log writings.
  * @param pool Apache pool
@@ -639,7 +648,7 @@ static mu_ent*
 mysql_cache_make_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	const mu_config *const conf, mu_ent_names *const names)
 {
-	MYSQL		*mysql;
+	MYSQL		*mysql = conf->cache_setup;
 	MYSQL_RES	*mysql_res = NULL;
 	MYSQL_ROW	mysql_row;
 	mu_ent		*p = NULL;
@@ -647,20 +656,11 @@ mysql_cache_make_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	char		*dirs = NULL, *bases = NULL, *dirn = NULL, *basen = NULL;
 	struct stat	statbuf;
 
-	if (unlikely((!conf->cache_setup) || (!mysql_params.ready)))
+	if (unlikely((!mysql) || (!mysql_params.ready)))
 		return p;
 	
 	if (unlikely(stat(names->filename, &statbuf)))
 		return p;
-	
-	mysql = mysql_init(NULL);
-	
-	if (unlikely(!mysql_real_connect(mysql, mysql_params.host, mysql_params.user,
-			mysql_params.pass, mysql_params.db, 0, NULL, 0)))
-		goto error;
-	    
-	if (unlikely(mysql_set_character_set(mysql, "utf8")))
-		goto error;
 
 	dirs = strdup(names->filename);
 	bases = strdup(names->filename);
@@ -678,6 +678,7 @@ mysql_cache_make_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	mysql_real_escape_string(mysql, sqldirn, dirn, strlen(dirn));
 	mysql_real_escape_string(mysql, sqlbasen, basen, strlen(basen));
 
+	/* XXX MEM OPTIM: asprintf() */
 	query = apr_psprintf(pool, "SELECT " SQL_CDATAF " FROM `" TABLE_FILES
 			     "` WHERE `pid`=(SELECT `id` from `" TABLE_DIRS "` WHERE `fullpath`='%s')"
 			     "AND `filename`='%s' COLLATE utf8_bin",
@@ -714,7 +715,6 @@ exit:
 	mysql_free_result(mysql_res);
 	free(dirs), free(bases);
 	free(sqldirn), free(sqlbasen);
-	mysql_close(mysql);
 	return p;
 }
 
@@ -735,25 +735,16 @@ static void
 mysql_cache_write(request_rec *r, const mu_ent *const p,
 	const mu_config *const conf, const mu_ent_names *const names)
 {
-	MYSQL		*mysql = NULL;
+	MYSQL		*mysql = conf->cache_setup;
 	MYSQL_RES	*mysql_res = NULL;
 	char		* restrict sqldirn = NULL, * restrict sqlbasen = NULL, * restrict query = NULL;
 	char		* restrict sqlalb = NULL, * restrict sqlart = NULL, * restrict sqltit = NULL, * restrict sqlgen = NULL;
 	char		*dirs = NULL, *bases = NULL, *dirn = NULL, *basen = NULL;
 	unsigned int	myerrno = 0;
 	
-	if (unlikely((!conf->cache_setup) || (!mysql_params.ready)))
+	if (unlikely((!mysql) || (!mysql_params.ready)))
 		return;
 
-	mysql = mysql_init(NULL);
-	
-	if (unlikely(!mysql_real_connect(mysql, mysql_params.host, mysql_params.user,
-			mysql_params.pass, mysql_params.db, 0, NULL, 0)))
-		goto error;
-	
-	if (unlikely(mysql_set_character_set(mysql, "utf8")))
-		goto error;
-	
 	/* if we have a directory, it could be a child dir in a given directory listing
 	 * we need to record it so that mysql_cache_dircontents() works properly, but
 	 * we also need to differentiate from normally created dirs by making sure that
@@ -899,7 +890,6 @@ exit:
 	free(dirs), free(bases);
 	free(sqldirn), free(sqlbasen);
 	free(sqlalb), free(sqlart), free(sqlgen), free(sqltit);
-	mysql_close(mysql);
 	return;
 }
 
@@ -961,7 +951,10 @@ static mysql_dir
 
 	/* this request should hopefully return only immediate subdirs,
 	as it will only match entries not containing any '/' after the current
-	path. Indeed, we always strip trailing '/' in mysql_cache_check_dir(). */
+	path. Indeed, we always strip trailing '/' in mysql_cache_check_dir().
+	XXX correct modifier for size_t is 'z' but seems (some versions of?)
+	apache doesn't know about it. */
+	/* XXX MEM OPTIM: asprintf() */
 	query = apr_psprintf(pool, "SELECT fullpath FROM " TABLE_DIRS " WHERE "
 				"`fullpath` LIKE '%s/%%' AND (LOCATE('/', fullpath, %lu)=0)",
 						 sqlstr, strlen(sqlstr)+2);
@@ -986,6 +979,7 @@ static mysql_dir
 
 	/* 2. */
 	/* we're reusing the previous content of sqlstr */
+	/* XXX MEM OPTIM: asprintf() */
 	query = apr_psprintf(pool, "SELECT " SQL_CDATAF ",filename FROM `" TABLE_FILES
 			"` WHERE `pid`="
 			"(SELECT `id` from `" TABLE_DIRS "` WHERE `fullpath`='%s')",
@@ -1020,6 +1014,7 @@ static mysql_dir
 	while ((mysql_row = mysql_fetch_row(mysql_res))) {
 		/* Check whether the cache is somehow invalid, if so, requeue the file */
 		if (unlikely(((unsigned)atoi(mysql_row[0]) < CACHE_VERS))) {
+			/* XXX MEM OPTIM: malloc here, free in readdir() */
 			basen = apr_pstrdup(pool, mysql_row[SQL_CDATAN]);
 			mdir = (mysql_dir *)apr_pcalloc(pool, sizeof(mysql_dir));
 			mdir->d_name = basen;
@@ -1107,6 +1102,7 @@ error:
  * elsewhere in the code (in particular in mysql_cache_make_entry()).
  *
  * @warning Asserts @a names->filename won't contain trailing /.
+ * @todo uses MI_QUICKPL, API should be fixed.
  *
  * @param r Apache request_rec struct to handle log writings and pool allocation.
  * @param pack A pack to add new entries to.
@@ -1120,24 +1116,17 @@ static void
 *mysql_cache_opendir(request_rec *r, mu_pack *const pack, const mu_config *const conf,
 			const mu_ent_names * const names, unsigned long soptions)
 {
-	MYSQL		*mysql;
+	MYSQL		*mysql = conf->cache_setup;
 	int		val;
 	const char *const path = names->filename;
 	mysql_dir	*mdir = NULL;
 	void		*ret = NULL;
 	struct stat	dirstat;
 
-	if (unlikely((!conf->cache_setup) || (!mysql_params.ready)))
-		goto error;
-
-	mysql = mysql_init(NULL);
-	
-	if (unlikely(!mysql_real_connect(mysql, mysql_params.host, mysql_params.user,
-			mysql_params.pass, mysql_params.db, 0, NULL, 0)))
-		goto error;
-
-	if (unlikely(mysql_set_character_set(mysql, "utf8")))
+	if (unlikely((!mysql) || (!mysql_params.ready))) {
+		mi_rdebug("FATAL: Entered with mysql=%x, mysql_params.ready=%d", mysql, mysql_params.ready);
 		goto error;
+	}
 
 	val = mysql_cache_check_dir(r, conf, path, mysql);
 	switch (val) {
@@ -1147,19 +1136,23 @@ static void
 			mysql_cache_remove_dir(r, path, mysql);
 		case CA_CREATE:
 		case CA_NOTREADY:
-			stat(path, &dirstat);
-			mysql_cache_make_dir(r,	path, dirstat.st_mtime, mysql);
+			if (!(conf->options & MI_QUICKPL)) {
+				/* XXX HACK. This is a gross workaround until the interface
+				 * is better specified: we don't want to record a directory
+				 * in the cache as valid when we're not going to record any
+				 * content for it, which happens when MI_QUICKPL is on */
+				stat(path, &dirstat);
+				mysql_cache_make_dir(r,	path, dirstat.st_mtime, mysql);
+			}
 		case CA_FATAL:
 		default:
-			goto out;
+			goto error;
 	}
 	
 	mdir = mysql_cache_dircontents(r, r->pool, pack, conf, names, mysql, soptions);
 	
 	ret = (void *)mdir;	/* NULL if mysql_cache_dircontents() failed, which is what we want */
 
-out:
-	mysql_close(mysql);
 error:
 	return ret;
 }
@@ -1190,23 +1183,89 @@ static const char
 	return ret;
 }
 
+/**
+ * Initializes the database connection.
+ *
+ * We really only want one database connection max per request.
+ *
+ * @param r Apache request_rec struct to handle log writings and pool allocation.
+ * @param conf The config structure used to store cache private data.
+ */
+static void
+mysql_cache_prologue(request_rec *r, mu_config *const conf)
+{
+	MYSQL	*mysql = NULL;
+
+	if (unlikely(!mysql_params.ready))
+		goto out;
+
+	mysql = mysql_init(NULL);
+	if (unlikely(!mysql))
+		goto out;
+
+	if (unlikely(!mysql_real_connect(mysql, mysql_params.host, mysql_params.user,
+					 mysql_params.pass, mysql_params.db, 0, NULL, 0))) {
+		mysql_close(mysql);
+		goto out;
+	}
+
+	if (unlikely(mysql_set_character_set(mysql, "utf8"))) {
+		mysql_close(mysql);
+		goto out;
+	}
+
+	conf->cache_setup = mysql;
+out:
+	return;
+}
+
+/**
+ * Closes the database connection.
+ *
+ * This is the counter part to the prologue(): we need to properly shutdown the
+ * database connection that we established there.
+ *
+ * @param r Apache request_rec struct to handle log writings and pool allocation.
+ * @param conf The config structure used to store cache private data.
+ */
+static void
+mysql_cache_epilogue(request_rec *r, mu_config *const conf)
+{
+	MYSQL	*mysql = (MYSQL *)conf->cache_setup;
+	
+	if (unlikely(!mysql))
+		goto out;
+
+	mysql_close(mysql);
+	conf->cache_setup = NULL;	
+out:
+	return;
+}
+
 static const cache_backend cache_backend_mysql = {
-	/* opendir */	mysql_cache_opendir,
-	/* readdir */	mysql_cache_readdir,
-	/* closedir */	NULL,
-	/* make_entry */mysql_cache_make_entry,
-	/* write */	mysql_cache_write
+	.opendir =	mysql_cache_opendir,
+	.readdir =	mysql_cache_readdir,
+	.closedir =	NULL,
+	.make_entry =	mysql_cache_make_entry,
+	.write =	mysql_cache_write,
+	.prologue =	mysql_cache_prologue,
+	.epilogue =	mysql_cache_epilogue,
 };
 
 int cache_mysql_setup(cmd_parms *cmd, const char *const setup_string, mu_config *const conf)
 {
 	static const char biniou[] = "mysql://";
+	int ret = 1;
 
 	if (strncmp(biniou, setup_string, 8) == 0) {
+		ret = mysql_cache_init(cmd->server, setup_string+8);
+		if (ret)
+			goto exit;
+		
+		conf->cache_setup = NULL;	/* this is prologue()'s baby */
 		conf->cache = &cache_backend_mysql;
-		conf->cache_setup = apr_pstrdup(cmd->pool, setup_string+8);
-		return mysql_cache_init(cmd->server, conf->cache_setup);
 	}
 
-	return 1;
+exit:
+	return ret;
 }
