@@ -2,11 +2,11 @@
  *  mod_musicindex.c
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.c 900 2009-12-06 21:48:19Z varenet $
+ *  $Id: mod_musicindex.c 934 2010-05-29 23:13:55Z varenet $
  *
  *  Created by Regis BOUDIN on Sat Dec 28 2002.
  *  Copyright (c) 2002-2006 Regis BOUDIN
- *  Copyright (c) 2002-2005,2007-2008 Thibaut VARENE
+ *  Copyright (c) 2002-2005,2007-2008,2010 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -29,16 +29,16 @@
  *	- Allow/deny file streaming.
  *	- Allow/deny file searching (recursively or not).
  *	- Default sorting order in Apache configuration and dynamic sorting while browsing.
- *	- Random directory on the fly relocation (EXPERIMENTAL).
+ *	- Random directory on the fly relocation.
  *	- Highly configurable data display.
  *	- Custom folder picture (using (.){cover,folder}.{png,jpg,gif} if found).
- *	- Cache subsystem (using flat files).
+ *	- Cache subsystem (using flat files or MySQL db).
  *	- Can redirect to an icecast server running in "staticdir" mode.
  *	- Custom cross-directories playlists.
  *	- Multiple CSS support.
  *	- Per directory configuration (using .htaccess).
  *	- XHTML output.
- *	- Archive download (with libarchive - EXPERIMENTAL).
+ *	- Archive download.
  *	- RSS and Podcasts feeds generation.
  *	- Support for auth_basic authentication in URL generation
  *
@@ -51,6 +51,7 @@
  *	- <a href="http://flac.sourceforge.net/">libflac</a>.
  *	- <a href="http://code.google.com/p/mp4v2/">libmp4v2</a>.
  *	- <a href="http://people.freebsd.org/~kientzle/libarchive/">libarchive</a>.
+ *	- <a href="http://dev.mysql.com/downloads/#connector-c">libmysqlclient</a>.
  *
  * @subpage devnotes
  */
@@ -62,12 +63,12 @@
  *	things with grace and proper documentation ;)
  *
  *	The coding style used throughout the code tries to stick as much as
- *	possible to the GNU Coding Style and/or the Linux Kernel Coding Style.
+ *	possible to the Linux Kernel Coding Style.
  *
  *	Developers should also take special care to typing. Strict typing is the
  *	first protection against coding mistakes that might lead to dreadful
  *	crashes. The same applies to function modifier (in particular the
- *	@c static modifier, that should be used whenever appropriate.
+ *	@c static modifier, that should be used whenever appropriate).
  *
  *	Finally, we try to put Apache's specific arguments to functions at the
  *	beginning of their param list.
@@ -128,8 +129,9 @@
  * This file is the core of the module. It contains Apache's mandatory stuff.
  *
  * @author Regis Boudin
- * @version $Revision: 900 $
+ * @version $Revision: 934 $
  * @date 2002-2008
+ * @date 2010
  *
  * http://httpd.apache.org/docs-2.0/developer/API.html
  * http://httpd.apache.org/dev/apidoc/
@@ -145,10 +147,6 @@
  * @todo get rid of str*cmp() whenever possible.
  * @todo review the randomdir implementation.
  *
- * @todo redesign cache integration to speed it up:
- *	- cache access init (eg: db connex) on conn request
- *	- cache access cleanup (eg: db close) on end conn request
- *
  * @todo Check how much overhead a DAAP implementation would bring. The
  *	protocol works on top of HTTP (where we are) and could make use of the
  *	cache.
@@ -160,6 +158,9 @@
  * @todo Decide how long we want to support Apache 1.3. APR provides lots of
  *	useful features, and the API compatibility between 1.3 and 2.2 is not
  *	certain.
+ *
+ * @todo Check whether it'd make sense to abstract Apache's primitives so as
+ *	to make the module portable to other webservers.
  */
 
 #include "mod_musicindex.h"
@@ -303,6 +304,10 @@ static int handle_musicindex(request_rec *r)
 		return HTTP_MOVED_TEMPORARILY;
 	}
 
+	/* Cache prologue can happen no later than here */
+	if (conf->cache && conf->cache->prologue)
+		conf->cache->prologue(r, conf);
+
 	/* build the string containing the cookie content. We need it for
 	   webpages and requests to stream the cookie content */
 	if (((conf->options & (MI_STREAM|MI_TARBALL)) == 0) || ((conf->options & MI_STREAMRQ) == MI_STREAMCOOKIE) || ((conf->options & MI_DWNLDRQ) == MI_DWNLDCOOKIE))
@@ -324,6 +329,10 @@ static int handle_musicindex(request_rec *r)
 	/* Build custom link list, from conf->custom_list string */
 	if (((conf->options & MI_STREAM) == 0) && (conf->custom_list != NULL))
 		build_custom_list(r, &custom_pack, conf);
+	
+	/* Cache epilogue can happen as early as here */
+	if (conf->cache && conf->cache->epilogue)
+		conf->cache->epilogue(r, conf);
 
 	/* Depending on the request type, set the HTTP headers. By default we
 	   return a web page. */
@@ -344,9 +353,11 @@ static int handle_musicindex(request_rec *r)
 		apr_table_setn(r->headers_out, "Content-Disposition",
 			"filename = \"playlist.tar\"");
 		/* send the size of the archive so that browsers can put a nice
-		   progress bar */
+		   progress bar
+		   XXX correct modifier for size_t is 'z' but seems (some versions of?)
+		   apache doesn't know about it. */
 		apr_table_setn(r->headers_out, "Content-Length",
-			apr_psprintf(r->pool, "%ld", tballsize));
+			apr_psprintf(r->pool, "%lu", tballsize));
 	}
 #endif
 	else if (conf->options & MI_RSS) {
@@ -453,7 +464,14 @@ static int handle_musicfile(request_rec *r)
 		if (r->header_only)
 			return OK;
 
+		if (conf->cache && conf->cache->prologue)
+			conf->cache->prologue(r, conf);
+
 		make_music_entry(r, r->pool, &master_pack, conf, NULL, MI_ALLOWSTREAM);
+
+		if (conf->cache && conf->cache->epilogue)
+			conf->cache->epilogue(r, conf);
+
 		master_pack.fhead = master_pack.head;
 		send_playlist(r, &master_pack, conf);
 
