@@ -2,11 +2,11 @@
  *  mod_musicindex.h
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.h 907 2010-01-08 13:32:36Z varenet $
+ *  $Id: mod_musicindex.h 937 2010-06-03 14:36:08Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
- *  Copyright (c) 2003-2009 Thibaut VARENE
+ *  Copyright (c) 2003-2010 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -210,6 +210,7 @@ enum {
 #define MAX_STRING	1024		/**< Common reference for maximum string size. */
 #define MAX_GENRE	64		/**< max length for genre string. longest seen so far: 37 */
 #define MAX_FNAME	255		/**< max length for filenames. according to http://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits 255 bytes/chars seems a common limit */
+#define MAX_PATHNAME	4096		/**< max length for pathnames. according to http://en.wikipedia.org/wiki/Comparison_of_file_systems#cite_note-note-12-9 Linux has a limit of 4,096, which looks sensible */
 
 #define MI_LOG_PREFIX	"[mod_musicindex] "	/**< String used to prefix log messages */
 
@@ -333,11 +334,10 @@ typedef struct mu_config mu_config;
 
 /**
  * Cache backend structure, defining a standard set of operations that all
- * cache backends must implement.
+ * cache backends may implement.
  */
 typedef struct {
-	/** This function is the entry point into the cache backend system.
-	    It can be used in many weird ways, but is mostly expected to either
+	/** It can be used in many weird ways, but is mostly expected to either
 	    return NULL if the directory backend can't provide a full directory
 	    listing, and otherwise populate the entries with whatever is in the
 	    cache. If return is !NULL, (cache_backend::readdir()) will be
@@ -367,6 +367,14 @@ typedef struct {
 	/** This call tells the cache backend to write metadata for a given file. */
 	void (*write)(request_rec *r, const mu_ent *const entry, const mu_config *const conf,
 				  const mu_ent_names * const names);
+
+	/** This call is guaranteed to run before any other cache backend function
+	    is called anywhere in the code path. */ 
+	void (*prologue)(request_rec *r, mu_config *const conf);
+
+	/** This call is guaranteed to run after any other cache backend function
+	    is called in the current module execution */
+	void (*epilogue)(request_rec *r, mu_config *const conf);
 } cache_backend;
 
 /** Configuration handling structure. We use it to share configuration
@@ -391,10 +399,10 @@ struct mu_config {
 
 	const char * restrict iceserver;	/**< adress of an icecast server */
 
-	const char * restrict custom_list;
+	const char * restrict custom_list;	/**< custom playlist */
 
-	const cache_backend * restrict cache;
-	void * restrict cache_setup;
+	const cache_backend * restrict cache;	/**< registered cache backend (if any) */
+	void * restrict cache_setup;	/**< cache private data (if any) */
 
 	signed short dir_per_line;	/**< number of directories to display per line. If <0 disable "pretty folders" */
 };
